"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigBase = void 0;
var Cache = require("./Cache");
var DefaultLogger_1 = require("./DefaultLogger");
var ConsoleLogger_1 = require("./ConsoleLogger");
var pckg = require('../package.json');
var ConfigBase = /** @class */ (function () {
    function ConfigBase(_sdkKey, _options) {
        this.consoleLog = false;
        this.baseUrl = "https://cdn.floodgate.io";
        this.API_VERSION = "v1";
        if (!_sdkKey) {
            throw new Error("Invalid SDK Key");
        }
        this.sdkKey = _sdkKey;
        this.logger = new DefaultLogger_1.default();
        this.Version = pckg.version;
        if (_options) {
            if (!_options.cache) {
                // this.cache = new Cache.InMemoryCache();
                this.cache = new Cache.LocalStorageCache();
            }
            if (_options.consoleLog) {
                this.logger = new ConsoleLogger_1.default();
                this.logger.Log('Enabling console logging');
            }
            if (_options.configUrl) {
                this.configUrl = _options.configUrl;
            }
            if (_options.localConfigData) {
                this.localConfigData = _options.localConfigData;
            }
        }
        this.logger.Log("SDK Version = " + this.Version);
    }
    ConfigBase.prototype.buildUrl = function () {
        var url = this.baseUrl + "/environment-files/" + this.sdkKey + "/" + this.API_VERSION + "/flags-config.json";
        if (this.configUrl) {
            url = this.configUrl;
        }
        this.logger.Log("url = " + url);
        return url;
    };
    return ConfigBase;
}());
exports.ConfigBase = ConfigBase;
