"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Evaluator = void 0;
var Const = require("./Consts");
// @ts-ignore <ts(7016)>
var sha1 = require("js-sha1");
var Evaluator = /** @class */ (function () {
    function Evaluator(_logger) {
        this.logger = _logger;
    }
    Evaluator.prototype.Evaluate = function (_key, _json, _defaultValue, _user) {
        // @ts-ignore <ts(2339)>
        var flags = Object.values(_json);
        // @ts-ignore <ts(2769)>
        var flag = flags.find(function (flag) { return flag.key === _key; });
        if (flag === undefined || flag === null) {
            this.logger.Log("No flag data available, returning default SDK value: " + _defaultValue);
            // No flag found, return default value
            return _defaultValue;
        }
        if (_user === undefined || _user === null) {
            this.logger.Log("No user data available, returning default flag value: " + flag['value']);
            // If user is null then cannot evaluate targets or rollouts, return the flag default value
            return flag['value'];
        }
        if (!flag['is_targeting_enabled']) {
            // Evaluate rollouts
            if (flag['is_rollout']) {
                var rolloutResult = this.EvaluateRollout(_key, _user.Id, flag['rollouts'], flag['value']);
                this.logger.Log("Rollout data available, returning rollout value: " + rolloutResult);
                return rolloutResult;
            }
            this.logger.Log("No targeting or rollout data available, returning default flag value: " + flag['value']);
            return flag['value'];
        }
        if (flag['is_targeting_enabled']) {
            var targetResult = this.EvaluateTargets(_key, flag, _user, flag['value']);
            this.logger.Log("Target data available, returning target value: " + targetResult);
            return targetResult;
        }
        this.logger.Log("Returning default value: " + _defaultValue);
        return _defaultValue;
    };
    Evaluator.prototype.EvaluateTargets = function (_key, _flag, _user, _defaultValue) {
        // @ts-ignore <ts(2339)>
        var targets = Object.values(_flag['targets']);
        var value = _defaultValue;
        var ruleResult = false;
        var evaluates;
        var target;
        for (var _i = 0, targets_1 = targets; _i < targets_1.length; _i++) {
            target = targets_1[_i];
            evaluates = true;
            value = target['value'];
            if (target['rules']) {
                ruleResult = this.EvaluateRules(target['rules'], _user);
                evaluates = evaluates && ruleResult;
            }
            if (evaluates) {
                if (target['is_rollout']) {
                    return this.EvaluateRollout(_key, _user.Id, target['rollouts'], _defaultValue);
                }
                return value;
            }
        }
        if (_flag['is_rollout']) {
            return this.EvaluateRollout(_key, _user.Id, _flag['rollouts'], _defaultValue);
        }
        return _defaultValue;
    };
    Evaluator.prototype.EvaluateRules = function (_rules, _user) {
        // @ts-ignore <ts(2339)>
        var rules = Object.values(_rules);
        var result = false;
        var rule;
        for (var _i = 0, rules_1 = rules; _i < rules_1.length; _i++) {
            rule = rules_1[_i];
            var userAttributeValue = this.GetUserAttributeValue(rule['attribute'], _user);
            if (!userAttributeValue || userAttributeValue === undefined) {
                result = false;
                continue;
            }
            var comparator = rule['comparator'];
            var values = rule['values'];
            this.logger.Log("Comparator = " + comparator);
            switch (comparator) {
                case Const.COMPARATOR_EQUAL_TO:
                    this.logger.Log("Comparator = COMPARATOR_EQUAL_TO : " + Const.COMPARATOR_EQUAL_TO);
                    result = values.map(function (v) { return v.toLowerCase(); })
                        .includes(userAttributeValue.toLowerCase());
                    break;
                case Const.COMPARATOR_NOT_EQUAL_TO:
                    this.logger.Log("Comparator = COMPARATOR_NOT_EQUAL_TO : " + Const.COMPARATOR_NOT_EQUAL_TO);
                    result = !values.includes(userAttributeValue.toLowerCase());
                    break;
                case Const.COMPARATOR_GREATER:
                    this.logger.Log("Comparator = COMPARATOR_GREATER : " + Const.COMPARATOR_GREATER);
                    result = false;
                    // If there are more than one values, take only the first one
                    if (!values || values.length == 0)
                        break;
                    var greater = values[0];
                    if (Number(userAttributeValue) > Number(greater)) {
                        result = true;
                    }
                    break;
                case Const.COMPARATOR_GREATER_EQUAL_TO:
                    this.logger.Log("Comparator = COMPARATOR_GREATER_EQUAL_TO : " + Const.COMPARATOR_GREATER_EQUAL_TO);
                    result = false;
                    // If there are more than one values, take only the first one
                    if (!values || values.length == 0)
                        break;
                    var greaterEqual = values[0];
                    if (Number(userAttributeValue) >= Number(greaterEqual)) {
                        result = true;
                    }
                    break;
                case Const.COMPARATOR_LESS:
                    this.logger.Log("Comparator = COMPARATOR_LESS : " + Const.COMPARATOR_LESS);
                    result = false;
                    // If there are more than one values, take only the first one
                    if (!values || values.length == 0)
                        break;
                    var less = values[0];
                    if (Number(userAttributeValue) < Number(less)) {
                        result = true;
                    }
                    break;
                case Const.COMPARATOR_LESS_EQUAL_TO:
                    this.logger.Log("Comparator = COMPARATOR_LESS_EQUAL_TO : " + Const.COMPARATOR_LESS_EQUAL_TO);
                    result = false;
                    // If there are more than one values, take only the first one
                    if (!values || values.length == 0)
                        break;
                    var lessEqual = values[0];
                    if (Number(userAttributeValue) <= Number(lessEqual)) {
                        result = true;
                    }
                    break;
                case Const.COMPARATOR_CONTAINS:
                    this.logger.Log("Comparator = COMPARATOR_CONTAINS : " + Const.COMPARATOR_CONTAINS);
                    result = false;
                    for (var _a = 0, values_1 = values; _a < values_1.length; _a++) {
                        var value = values_1[_a];
                        result = userAttributeValue.toLowerCase().includes(value.toLowerCase());
                        if (result)
                            break;
                    }
                    break;
                case Const.COMPARATOR_NOT_CONTAIN:
                    this.logger.Log("Comparator = COMPARATOR_NOT_CONTAIN : " + Const.COMPARATOR_NOT_CONTAIN);
                    var valid = true;
                    for (var _b = 0, values_2 = values; _b < values_2.length; _b++) {
                        var value = values_2[_b];
                        result = !userAttributeValue.toLowerCase().includes(value.toLowerCase());
                        valid = result && valid;
                    }
                    result = valid;
                    break;
                case Const.COMPARATOR_ENDS_WITH:
                    this.logger.Log("Comparator = COMPARATOR_ENDS_WITH : " + Const.COMPARATOR_ENDS_WITH);
                    result = false;
                    for (var _c = 0, values_3 = values; _c < values_3.length; _c++) {
                        var value = values_3[_c];
                        result = userAttributeValue.toLowerCase().endsWith(value.toLowerCase());
                        if (result)
                            break;
                    }
                    break;
                default:
                    result = false;
                    break;
            }
        }
        return result;
    };
    Evaluator.prototype.EvaluateRollout = function (_key, _userId, _rollouts, _defaultValue) {
        var rolloutValue = _defaultValue;
        var rolloutLowerLimit = 0;
        var rolloutUpperLimit = 0;
        var scale = this.GetScale(_key, _userId);
        for (var _i = 0, _rollouts_1 = _rollouts; _i < _rollouts_1.length; _i++) {
            var rollout = _rollouts_1[_i];
            rolloutUpperLimit = rolloutUpperLimit + parseInt(rollout['percentage']);
            if (scale > rolloutLowerLimit && scale <= rolloutUpperLimit) {
                rolloutValue = rollout['value'];
                break;
            }
            rolloutLowerLimit = rolloutLowerLimit + parseInt(rollout['percentage']);
        }
        return rolloutValue;
    };
    Evaluator.prototype.GetScale = function (_key, _userId) {
        var hashString = _key + _userId;
        var hash = sha1.hex(hashString).substring(0, 7);
        var scale = parseInt(hash, 16) % 100;
        return scale;
    };
    Evaluator.prototype.GetUserAttributeValue = function (_key, _user) {
        if (_key.toLowerCase() == "email") {
            return _user.Email;
        }
        if (_key.toLowerCase() == "name") {
            return _user.Name;
        }
        if (_user.CustomAttributes && _user.CustomAttributes[_key.toLowerCase()]) {
            return String(_user.CustomAttributes[_key.toLowerCase()]);
        }
        return null;
    };
    return Evaluator;
}());
exports.Evaluator = Evaluator;
